import type { SelectProps } from 'ant-design-vue'
import { Checkbox, Tag } from 'ant-design-vue'
import type { BasicColumn, FormProps } from '@/components/Table'
import type { VxeGridPropTypes } from '@/components/VxeTable'
import { Icon } from '@/components/Icon'

/**
 * 支持的数据源类型
 */
export const databaseTypeOptions: SelectProps['options'] = [
    {
        label: '常规表',
        value: 'General',
        options: [
            {
                label: '常规单表',
                value: 'SingleGeneral',
            },
            {
                label: '常规主表',
                value: 'MasterGeneral',
                disabled: false,
            },
            {
                label: '常规从表',
                value: 'SlaveGeneral',
                disabled: false,
            },
        ],
    },
    {
        label: '时序表',
        value: 'TimeSeries',
        options: [
            {
                label: '时序单表',
                value: 'SingleTimeSeries',
                disabled: true,
            },
            {
                label: '业务时序表',
                value: 'BusinessTimeSeries',
                disabled: true,
            },
        ],
    },
]

// ================================
export function getColumns(): BasicColumn[] {
    return [
        {
            title: '表类型',
            dataIndex: 'type',
        },
        {
            title: '表名',
            dataIndex: 'name',
        },
        {
            title: '表描述',
            dataIndex: 'describe',
        },
        {
            title: '版本',
            dataIndex: 'version',
            sorter: true,
        },
        {
            title: '同步状态',
            dataIndex: 'sync',
            sorter: true,
            customRender: ({ value }) => <Tag color={value ? 'green' : 'red'}>{value ? '已同步' : '未同步'}</Tag>,
        },
        {
            title: '创建时间',
            dataIndex: 'createTime',
            sorter: true,
            width: 180,
        },
    ]
}
export function getDataSource() {
    return (() => {
        const data = []

        data.push({
            id: '1',
            type: '单表',
            name: 'bs_staff',
            describe: '人员信息表',
            version: '1',
            sync: true,
            createTime: '2023-02-26 15:36:01',
        })

        return data
    })()
}
export function getFormConfig(): Partial<FormProps> {
    return {
        layout: 'inline',
        name: 'query',
        colon: true,
        compact: true,
        schemas: [
            {
                field: `type`,
                label: `表类型`,
                component: 'Select',
                componentProps: {
                    options: databaseTypeOptions,
                    mode: 'multiple',
                    style: {
                        width: '180px',
                    },
                },
            },
            {
                field: `name`,
                label: `表名`,
                component: 'Input',
            },
            {
                field: `describe`,
                label: `表描述`,
                component: 'Input',
            },
        ],
        showAdvancedButton: false,
        submitButtonOptions: {
            preIcon: 'material-symbols:search',
        },
    }
}

// ================================
export type FieldType =
    | 'Integer'
    | 'Float'
    | 'Double'
    | 'BigDecimal'
    | 'Char'
    | 'String'
    | 'Text'
    | 'Boolean'
    | 'LocalDate'
    | 'LocalTime'
    | 'LocalDateTime'
    | 'Point'
    | 'Line'
    | 'Polygon'
    | 'Geometry'
    | 'Blob'
    | 'JSONObject'
    | 'JSONArray'
/**
 * 支持的数据源类型
 */
export const fieldTypeOptions: SelectProps['options'] = [
    {
        label: '数值类型',
        value: 'Number',
        options: [
            {
                label: '整数',
                value: 'Integer',
            },
            {
                label: '单精度小数',
                value: 'Float',
            },
            {
                label: '双精度小数',
                value: 'Double',
            },
            {
                label: '高精度小数',
                value: 'BigDecimal',
            },
        ],
    },
    {
        label: '字符串类型',
        value: 'Varchar',
        options: [
            {
                label: '字符',
                value: 'Char',
            },
            {
                label: '字符串',
                value: 'String',
            },
            {
                label: '大文本',
                value: 'Text',
            },
        ],
    },
    {
        label: '布尔类型',
        value: 'Bool',
        options: [
            {
                label: '布尔',
                value: 'Boolean',
            },
        ],
    },
    {
        label: '日期类型',
        value: 'Date',
        options: [
            {
                label: '日期',
                value: 'LocalDate',
            },
            {
                label: '时间',
                value: 'LocalTime',
            },
            {
                label: '日期时间',
                value: 'LocalDateTime',
            },
        ],
    },
    {
        label: '空间类型',
        value: 'GIS',
        options: [
            {
                label: '坐标点',
                value: 'Point',
            },
            {
                label: '空间线',
                value: 'Line',
            },
            {
                label: '空间面',
                value: 'Polygon',
            },
            {
                label: '空间几何',
                value: 'Geometry',
            },
        ],
    },
    {
        label: '其他',
        value: 'Other',
        options: [
            {
                label: 'Blob 对象',
                value: 'Blob',
            },
            {
                label: 'JSON 对象',
                value: 'JSONObject',
            },
            {
                label: 'JSON 数组',
                value: 'JSONArray',
            },
        ],
    },
]

export interface DatabaseSchema {
    id?: string
    field?: string
    comment?: string
    type?: FieldType
    len?: number
    decimal?: number
    defaultValue?: string
    pk?: boolean
    empty?: boolean
    transient?: boolean
    editable?: boolean
    disabled?: boolean
    [key: string]: any
}

/**
 * 获取数据源字段列表
 * @returns Column[]
 */
export function getDatabaseSchemaColumns(change: () => void): VxeGridPropTypes.Columns {
    return [
        {
            type: 'html',
            align: 'center',
            width: 50,
            className: 'drag',
            slots: {
                default: () => {
                    return [<Icon icon="mdi:sort" />]
                },
            },
        },
        {
            type: 'checkbox',
            align: 'center',
            width: 50,
        },
        {
            type: 'seq',
            align: 'center',
            width: 50,
        },
        {
            title: '字段名称',
            field: 'field',
            editRender: {
                name: 'AInput',
                placeholder: '请点击输入',
                events: {
                    input: change,
                },
            },
        },
        {
            title: '字段备注',
            field: 'comment',
            editRender: {
                name: 'AInput',
                placeholder: '请点击输入',
                events: {
                    input: change,
                },
            },
        },
        {
            title: '字段类型',
            field: 'type',
            editRender: {
                name: 'ASelect',
                placeholder: '请点击选择',
                optionGroups: fieldTypeOptions,
                events: {
                    change,
                },
            },
        },
        {
            title: '字段长度',
            field: 'len',
            editRender: {
                name: 'AInputNumber',
                events: {
                    input: change,
                },
            },
        },
        {
            title: '小数点',
            field: 'decimal',
            editRender: {
                name: 'AInputNumber',
                events: {
                    input: change,
                },
            },
        },
        {
            title: '默认值',
            field: 'defaultValue',
            editRender: {
                name: 'AInput',
                events: {
                    input: change,
                },
            },
        },
        {
            title: '是否主键',
            field: 'pk',
            align: 'center',
            width: 100,
            slots: {
                default: ({ row }) => {
                    return [
                        <Checkbox
                            v-model:checked={row.pk}
                            disabled={row.disabled || row.disablePK}
                            onChange={change}
                        />,
                    ]
                },
            },
        },
        {
            title: '允许空值',
            field: 'empty',
            align: 'center',
            width: 100,
            slots: {
                default: ({ row }) => {
                    return [
                        <Checkbox
                            v-model:checked={row.empty}
                            disabled={row.disabled || row.disableEmpty}
                            onChange={change}
                        />,
                    ]
                },
            },
        },
        {
            title: '是否同步数据库',
            field: 'transient',
            align: 'center',
            width: 150,
            slots: {
                default: ({ row }) => {
                    return [
                        <Checkbox
                            v-model:checked={row.transient}
                            disabled={row.disabled || row.disableTransient}
                            onChange={change}
                        />,
                    ]
                },
            },
        },
        {
            title: '操作',
            align: 'center',
            fixed: 'right',
            width: 150,
            slots: { default: 'action' },
        },
    ]
}

export function getDatabaseForeignKeyColumns(change: () => void): VxeGridPropTypes.Columns {
    return [
        {
            type: 'seq',
            align: 'center',
            width: 50,
        },
        {
            title: '字段名称',
            field: 'field',
        },
        {
            title: '字段备注',
            field: 'comment',
        },
        {
            title: '字段类型',
            field: 'type',
        },
        {
            title: '外键表',
            field: 'foreignTable',
            editRender: {
                // TODO: 改用 Select 选择外键表，通过接口查询
                name: 'AInput',
                placeholder: '请点击输入',
                events: {
                    input: change,
                },
            },
        },
        {
            title: '外键字段',
            field: 'foreignField',
            editRender: {
                name: 'AInput',
                placeholder: '请点击输入',
                events: {
                    input: change,
                },
            },
        },
    ]
}

export type TableType = 'SingleGeneral' | 'MasterGeneral' | 'SlaveGeneral' | 'SingleTimeSeries' | 'BusinessTimeSeries'
export function getDefaultSchemaFieldDataSourcesByType(type: TableType): DatabaseSchema[] {
    if (type === 'SingleGeneral') {
        return [
            {
                id: 'id',
                field: 'id',
                comment: '主键',
                type: 'String',
                len: 32,
                pk: true,
                empty: false,
                transient: true,
                disabled: true,
            },
            {
                id: 'createBy',
                field: 'createBy',
                comment: '创建人',
                type: 'String',
                len: 32,
                pk: false,
                empty: false,
                transient: true,
            },
            {
                id: 'createTime',
                field: 'createTime',
                comment: '创建时间',
                type: 'LocalDateTime',
                len: 0,
                decimal: 0,
                pk: false,
                empty: false,
                transient: true,
            },
            {
                id: 'updateBy',
                field: 'updateBy',
                comment: '修改人',
                type: 'String',
                len: 32,
                pk: false,
                empty: false,
                transient: true,
            },
            {
                id: 'updateTime',
                field: 'updateTime',
                comment: '修改时间',
                type: 'LocalDateTime',
                len: 0,
                decimal: 0,
                pk: false,
                empty: false,
                transient: true,
            },
        ]
    }

    return []
}
